@@ -5,7 +5,10 @@ Configuration for Sphinx.
 extensions = [
     "sphinxcontrib.video",
     "sphinxnotes.strike",
+    "sphinxcontrib_text_styles",
     "sphinx_notion",
+    "sphinx_simplepdf",
     "sphinx_toolbox.collapse",
     "atsphinx.audioplayer",
+    "sphinx_immaterial.task_lists",
 ]
