@@ -5,14 +5,16 @@ Inspired by https://github.com/ftnext/sphinx-notion/blob/main/upload.py.
 
 import json
 import sys
+from enum import Enum
 from pathlib import Path
-from typing import Any
+from typing import TYPE_CHECKING, Any
 from urllib.parse import urlparse
 from urllib.request import url2pathname
 
 import click
 from beartype import beartype
 from ultimate_notion import Emoji, Session
+from ultimate_notion.blocks import PDF as UnoPDF  # noqa: N811
 from ultimate_notion.blocks import Audio as UnoAudio
 from ultimate_notion.blocks import Block
 from ultimate_notion.blocks import Image as UnoImage
@@ -20,6 +22,10 @@ from ultimate_notion.blocks import Video as UnoVideo
 from ultimate_notion.file import UploadedFile
 from ultimate_notion.obj_api.blocks import Block as UnoObjAPIBlock
 
+if TYPE_CHECKING:
+    from ultimate_notion.database import Database
+    from ultimate_notion.page import Page
+
 
 @beartype
 def _upload_local_file(
@@ -34,7 +40,9 @@ def _upload_local_file(
     if parsed.scheme != "file":
         return None
 
-    file_path = Path(url2pathname(pathname=parsed.path))
+    # Ignore ``mypy`` error as the keyword arguments are different across
+    # Python versions and platforms.
+    file_path = Path(url2pathname(parsed.path))  # type: ignore[misc]
     with file_path.open(mode="rb") as f:
         uploaded_file = session.upload(
             file=f,
@@ -57,7 +65,7 @@ def _block_from_details(
     """
     block = Block.wrap_obj_ref(UnoObjAPIBlock.model_validate(obj=details))
 
-    if isinstance(block, (UnoImage, UnoVideo, UnoAudio)):
+    if isinstance(block, (UnoImage, UnoVideo, UnoAudio, UnoPDF)):
         uploaded_file = _upload_local_file(url=block.url, session=session)
         if uploaded_file is not None:
             return block.__class__(file=uploaded_file, caption=block.caption)
@@ -65,6 +73,16 @@ def _block_from_details(
     return block
 
 
+@beartype
+class _ParentType(Enum):
+    """
+    Type of parent that new page will live under.
+    """
+
+    PAGE = "page"
+    DATABASE = "database"
+
+
 @click.command()
 @click.option(
     "--file",
@@ -78,10 +96,16 @@ def _block_from_details(
     ),
 )
 @click.option(
-    "--parent-page-id",
-    help="Parent page ID (integration connected)",
+    "--parent-id",
+    help="Parent page or database ID (integration connected)",
     required=True,
 )
+@click.option(
+    "--parent-type",
+    help="Parent type",
+    required=True,
+    type=click.Choice(choices=_ParentType, case_sensitive=False),
+)
 @click.option(
     "--title",
     help="Title of the page to update (or create if it does not exist)",
@@ -96,7 +120,8 @@ def _block_from_details(
 def main(
     *,
     file: Path,
-    parent_page_id: str,
+    parent_id: str,
+    parent_type: _ParentType,
     title: str,
     icon: str | None = None,
 ) -> None:
@@ -107,11 +132,17 @@ def main(
 
     blocks = json.loads(s=file.read_text(encoding="utf-8"))
 
-    parent_page = session.get_page(page_ref=parent_page_id)
+    parent: Page | Database
+    match parent_type:
+        case _ParentType.PAGE:
+            parent = session.get_page(page_ref=parent_id)
+            subpages = parent.subpages
+        case _ParentType.DATABASE:
+            parent = session.get_db(db_ref=parent_id)
+            subpages = parent.get_all_pages().to_pages()
+
     pages_matching_title = [
-        child_page
-        for child_page in parent_page.subpages
-        if child_page.title == title
+        child_page for child_page in subpages if child_page.title == title
     ]
 
     if pages_matching_title:
@@ -122,7 +153,7 @@ def main(
         assert len(pages_matching_title) == 1, msg
         (page,) = pages_matching_title
     else:
-        page = session.create_page(parent=parent_page, title=title)
+        page = session.create_page(parent=parent, title=title)
         sys.stdout.write(f"Created new page: {title} (ID: {page.id})\n")
 
     if icon:
