@@ -3,6 +3,7 @@ Sphinx Notion Builder.
 """
 
 import json
+import logging
 from dataclasses import dataclass
 from functools import singledispatch
 from pathlib import Path
@@ -17,15 +18,24 @@ from docutils import nodes
 from docutils.nodes import NodeVisitor
 from sphinx.application import Sphinx
 from sphinx.builders.text import TextBuilder
+from sphinx.util import docutils as sphinx_docutils
+from sphinx.util import logging as sphinx_logging
 from sphinx.util.typing import ExtensionMetadata
+from sphinx_immaterial.task_lists import (  # pyright: ignore[reportMissingTypeStubs]
+    checkbox_label,
+)
+from sphinx_simplepdf.directives.pdfinclude import (  # pyright: ignore[reportMissingTypeStubs]
+    PdfIncludeDirective,
+)
 from sphinx_toolbox.collapse import CollapseNode
 from sphinxcontrib.video import (  # pyright: ignore[reportMissingTypeStubs]
     video_node,
 )
 from sphinxnotes.strike import strike_node
 from ultimate_notion import Emoji
+from ultimate_notion.blocks import PDF as UnoPDF  # noqa: N811
 from ultimate_notion.blocks import Audio as UnoAudio
-from ultimate_notion.blocks import Block, ChildrenMixin
+from ultimate_notion.blocks import Block, ParentBlock
 from ultimate_notion.blocks import BulletedItem as UnoBulletedItem
 from ultimate_notion.blocks import Callout as UnoCallout
 from ultimate_notion.blocks import Code as UnoCode
@@ -51,14 +61,93 @@ from ultimate_notion.blocks import Table as UnoTable
 from ultimate_notion.blocks import (
     TableOfContents as UnoTableOfContents,
 )
+from ultimate_notion.blocks import ToDoItem as UnoToDoItem
 from ultimate_notion.blocks import (
     ToggleItem as UnoToggleItem,
 )
 from ultimate_notion.blocks import Video as UnoVideo
 from ultimate_notion.file import ExternalFile
-from ultimate_notion.obj_api.enums import BGColor, CodeLang
+from ultimate_notion.obj_api.enums import BGColor, CodeLang, Color
 from ultimate_notion.rich_text import Text, text
 
+_LOGGER = sphinx_logging.getLogger(name=__name__)
+
+
+@beartype
+def _color_from_node(*, node: nodes.inline) -> Color | None:
+    """Extract Notion color from CSS classes.
+
+    ``sphinxcontrib-text-styles`` creates classes like 'text-red', 'text-
+    blue', etc.
+    """
+    classes = node.attributes.get("classes", [])
+    color_mapping: dict[str, Color] = {
+        "text-red": Color.RED,
+        "text-blue": Color.BLUE,
+        "text-green": Color.GREEN,
+        "text-yellow": Color.YELLOW,
+        "text-orange": Color.ORANGE,
+        "text-purple": Color.PURPLE,
+        "text-pink": Color.PINK,
+        "text-brown": Color.BROWN,
+        "text-gray": Color.GRAY,
+        "text-grey": Color.GRAY,
+    }
+
+    for css_class in classes:
+        if css_class in color_mapping:
+            return color_mapping[css_class]
+
+        # This warning being here assumes that only color classes,
+        # and only classes from ``sphinxcontrib-text-styles``, are used.
+        _LOGGER.warning(
+            "Unsupported text style classes: %s. "
+            "Text will be rendered without styling.",
+            css_class,
+        )
+
+    return None
+
+
+@beartype
+def _serialize_block_with_children(
+    *,
+    block: Block,
+) -> dict[str, Any]:
+    """
+    Convert a block to a JSON-serializable format which includes its children.
+    """
+    serialized_obj = block.obj_ref.serialize_for_api()
+    if isinstance(block, ParentBlock) and block.children:
+        serialized_obj[block.obj_ref.type]["children"] = [
+            _serialize_block_with_children(block=child)
+            for child in block.children
+        ]
+    return serialized_obj
+
+
+@beartype
+class _PdfNode(nodes.raw):  # pylint: disable=too-many-ancestors
+    """
+    Custom PDF node for Notion PDF blocks.
+    """
+
+
+@beartype
+class _NotionPdfIncludeDirective(PdfIncludeDirective):
+    """
+    PDF include directive that creates Notion PDF blocks.
+    """
+
+    def run(self) -> list[nodes.raw]:
+        """
+        Create a Notion PDF block.
+        """
+        (pdf_file,) = self.arguments
+        node = _PdfNode()
+        node.attributes["uri"] = pdf_file
+        return [node]
+
 
 @dataclass
 class _TableStructure:
@@ -96,6 +185,15 @@ def _create_rich_text_from_children(*, node: nodes.Element) -> Text:
             )
         elif isinstance(child, nodes.target):
             continue
+        elif isinstance(child, nodes.inline):
+            new_text = text(
+                text=child.astext(),
+                bold=isinstance(child, nodes.strong),
+                italic=isinstance(child, nodes.emphasis),
+                code=isinstance(child, nodes.literal),
+                strikethrough=isinstance(child, strike_node),
+                color=_color_from_node(node=child),
+            )
         else:
             new_text = text(
                 text=child.astext(),
@@ -123,8 +221,10 @@ def _extract_table_structure(
 
     # In Notion, all rows must have the same number of columns.
     # Therefore there is only one ``tgroup``.
-    (tgroup,) = node.children
-    assert isinstance(tgroup, nodes.tgroup)
+    tgroups = [
+        child for child in node.children if isinstance(child, nodes.tgroup)
+    ]
+    (tgroup,) = tgroups
 
     for tgroup_child in tgroup.children:
         if isinstance(tgroup_child, nodes.colspec):
@@ -229,6 +329,9 @@ def _map_pygments_to_notion_language(*, pygments_lang: str) -> CodeLang:
         "graphql": CodeLang.GRAPHQL,
         "groovy": CodeLang.GROOVY,
         "haskell": CodeLang.HASKELL,
+        # This is not a perfect match, but at least JSON within the
+        # HTTP definition will be highlighted.
+        "http": CodeLang.JSON,
         "html": CodeLang.HTML,
         "java": CodeLang.JAVA,
         "javascript": CodeLang.JAVASCRIPT,
@@ -299,77 +402,6 @@ def _map_pygments_to_notion_language(*, pygments_lang: str) -> CodeLang:
     return language_mapping[pygments_lang.lower()]
 
 
-@beartype
-def _process_list_item_recursively(
-    *,
-    node: nodes.list_item,
-) -> list[Block]:
-    """
-    Recursively process a list item node and return a BulletedItem.
-    """
-    paragraph = node.children[0]
-    assert isinstance(paragraph, nodes.paragraph)
-    rich_text = _create_rich_text_from_children(node=paragraph)
-    block = UnoBulletedItem(text=rich_text)
-
-    assert isinstance(node, nodes.list_item)
-
-    for child in node.children[1:]:
-        if not isinstance(child, nodes.bullet_list):
-            bullet_only_msg = (
-                "The only thing Notion supports within a bullet list is a "
-                f"bullet list. Given {type(child).__name__} on line "
-                f"{child.line} in {child.source}"
-            )
-            # Ignore error which is about a type error, but we want to
-            # raise a value error because the user has not sent anything to
-            # do with types.
-            raise ValueError(bullet_only_msg)  # noqa: TRY004
-        for nested_list_item in child.children:
-            assert isinstance(nested_list_item, nodes.list_item)
-            block.append(
-                blocks=_process_list_item_recursively(
-                    node=nested_list_item,
-                )
-            )
-    return [block]
-
-
-@beartype
-def _process_numbered_list_item_recursively(
-    *,
-    node: nodes.list_item,
-) -> list[Block]:
-    """
-    Recursively process a numbered list item node and return a NumberedItem.
-    """
-    paragraph = node.children[0]
-    assert isinstance(paragraph, nodes.paragraph)
-    rich_text = _create_rich_text_from_children(node=paragraph)
-    block = UnoNumberedItem(text=rich_text)
-
-    numbered_only_msg = (
-        "The only thing Notion supports within a numbered list is a "
-        f"numbered list. Given {type(node).__name__} on line {node.line} "
-        f"in {node.source}"
-    )
-    assert isinstance(node, nodes.list_item)
-
-    for child in node.children[1:]:
-        assert isinstance(child, nodes.enumerated_list), numbered_only_msg
-        for nested_list_item in child.children:
-            assert isinstance(nested_list_item, nodes.list_item), (
-                numbered_only_msg
-            )
-
-            block.append(
-                blocks=_process_numbered_list_item_recursively(
-                    node=nested_list_item,
-                )
-            )
-    return [block]
-
-
 @singledispatch
 @beartype
 def _process_node_to_blocks(
@@ -384,6 +416,7 @@ def _process_node_to_blocks(
     raise NotImplementedError(node)
 
 
+@beartype
 @_process_node_to_blocks.register
 def _(
     node: nodes.table,
@@ -404,10 +437,7 @@ def _(
                 f"{child.line} in {child.source}, but Notion tables "
                 "do not have titles."
             )
-            # Ignore error which is about a type error, but we want to
-            # raise a value error because the user has not sent anything to
-            # do with types.
-            raise ValueError(table_no_titles_msg)  # noqa: TRY004
+            _LOGGER.warning(msg=table_no_titles_msg)
 
     table_structure = _extract_table_structure(node=node)
 
@@ -426,14 +456,14 @@ def _(
             f"{first_header_row_paragraph.source}, last header row is on "
             f"line {last_header_row_line}"
         )
-        raise ValueError(table_multiple_header_rows_msg)
+        _LOGGER.warning(msg=table_multiple_header_rows_msg)
 
     if table_structure.num_stub_columns > 1:
         table_more_than_one_stub_column_msg = (
             f"Tables with more than 1 stub column are not supported. "
             f"Found {table_structure.num_stub_columns} stub columns."
         )
-        raise ValueError(table_more_than_one_stub_column_msg)
+        _LOGGER.warning(msg=table_more_than_one_stub_column_msg)
 
     rows = [*table_structure.header_rows, *table_structure.body_rows]
     table = UnoTable(
@@ -454,6 +484,7 @@ def _(
     return [table]
 
 
+@beartype
 @_process_node_to_blocks.register
 def _(
     node: nodes.paragraph,
@@ -468,6 +499,7 @@ def _(
     return [UnoParagraph(text=rich_text)]
 
 
+@beartype
 @_process_node_to_blocks.register
 def _(
     node: nodes.block_quote,
@@ -482,6 +514,7 @@ def _(
     return [UnoQuote(text=rich_text)]
 
 
+@beartype
 @_process_node_to_blocks.register
 def _(
     node: nodes.literal_block,
@@ -500,6 +533,7 @@ def _(
     return [UnoCode(text=code_text, language=language)]
 
 
+@beartype
 @_process_node_to_blocks.register
 def _(
     node: nodes.bullet_list,
@@ -509,18 +543,44 @@ def _(
     """
     Process bullet list nodes by creating Notion BulletedItem blocks.
     """
-    del section_level
     result: list[Block] = []
     for list_item in node.children:
         assert isinstance(list_item, nodes.list_item)
-        result.extend(
-            _process_list_item_recursively(
-                node=list_item,
+        first_child = list_item.children[0]
+        if isinstance(first_child, nodes.paragraph):
+            paragraph = first_child
+            rich_text = _create_rich_text_from_children(node=paragraph)
+            bulleted_item_block = UnoBulletedItem(text=rich_text)
+
+            for child in list_item.children[1:]:
+                child_blocks = _process_node_to_blocks(
+                    child,
+                    section_level=section_level,
+                )
+                bulleted_item_block.append(blocks=child_blocks)
+            result.append(bulleted_item_block)
+        else:
+            assert isinstance(first_child, checkbox_label)
+            label_text_node = list_item.children[1]
+            # Get the checked state from the checkbox_label node
+            checked = first_child.attributes.get("checked", False)
+            assert isinstance(label_text_node, nodes.paragraph)
+            rich_text = _create_rich_text_from_children(
+                node=label_text_node,
             )
-        )
+            todo_item_block = UnoToDoItem(text=rich_text, checked=checked)
+
+            for child in list_item.children[2:]:
+                child_blocks = _process_node_to_blocks(
+                    child,
+                    section_level=section_level,
+                )
+                todo_item_block.append(blocks=child_blocks)
+            result.append(todo_item_block)
     return result
 
 
+@beartype
 @_process_node_to_blocks.register
 def _(
     node: nodes.enumerated_list,
@@ -528,25 +588,47 @@ def _(
     section_level: int,
 ) -> list[Block]:
     """
-    Process enumerated list nodes by creating Notion NumberedItem blocks.
+    Process enumerated list nodes by creating Notion NumberedItem or ToDoItem
+    blocks.
     """
-    del section_level
     result: list[Block] = []
-    numbered_only_msg = (
-        "The only thing Notion supports within a numbered list is a "
-        f"numbered list. Given {type(node).__name__} on line {node.line} "
-        f"in {node.source}"
-    )
     for list_item in node.children:
-        assert isinstance(list_item, nodes.list_item), numbered_only_msg
-        result.extend(
-            _process_numbered_list_item_recursively(
-                node=list_item,
+        assert isinstance(list_item, nodes.list_item)
+        first_child = list_item.children[0]
+        if isinstance(first_child, nodes.paragraph):
+            paragraph = first_child
+            rich_text = _create_rich_text_from_children(node=paragraph)
+            block = UnoNumberedItem(text=rich_text)
+
+            for child in list_item.children[1:]:
+                child_blocks = _process_node_to_blocks(
+                    child,
+                    section_level=section_level,
+                )
+                block.append(blocks=child_blocks)
+            result.append(block)
+        else:
+            assert isinstance(first_child, checkbox_label)
+            label_text_node = list_item.children[1]
+            # Get the checked state from the checkbox_label node
+            checked = first_child.attributes.get("checked", False)
+            assert isinstance(label_text_node, nodes.paragraph)
+            rich_text = _create_rich_text_from_children(
+                node=label_text_node,
             )
-        )
+            todo_item_block = UnoToDoItem(text=rich_text, checked=checked)
+
+            for child in list_item.children[2:]:
+                child_blocks = _process_node_to_blocks(
+                    child,
+                    section_level=section_level,
+                )
+                todo_item_block.append(blocks=child_blocks)
+            result.append(todo_item_block)
     return result
 
 
+@beartype
 @_process_node_to_blocks.register
 def _(
     node: nodes.topic,
@@ -563,6 +645,7 @@ def _(
     return [UnoTableOfContents()]
 
 
+@beartype
 @_process_node_to_blocks.register
 def _(
     node: nodes.compound,
@@ -580,6 +663,7 @@ def _(
     return []
 
 
+@beartype
 @_process_node_to_blocks.register
 def _(
     node: nodes.title,
@@ -649,6 +733,7 @@ def _create_admonition_callout(
     return [block]
 
 
+@beartype
 @_process_node_to_blocks.register
 def _(
     node: nodes.note,
@@ -666,6 +751,7 @@ def _(
     )
 
 
+@beartype
 @_process_node_to_blocks.register
 def _(
     node: nodes.warning,
@@ -683,6 +769,7 @@ def _(
     )
 
 
+@beartype
 @_process_node_to_blocks.register
 def _(
     node: nodes.tip,
@@ -700,6 +787,155 @@ def _(
     )
 
 
+@beartype
+@_process_node_to_blocks.register
+def _(
+    node: nodes.attention,
+    *,
+    section_level: int,
+) -> list[Block]:
+    """
+    Process attention admonition nodes by creating Notion Callout blocks.
+    """
+    del section_level
+    return _create_admonition_callout(
+        node=node,
+        emoji="👀",
+        background_color=BGColor.YELLOW,
+    )
+
+
+@beartype
+@_process_node_to_blocks.register
+def _(
+    node: nodes.caution,
+    *,
+    section_level: int,
+) -> list[Block]:
+    """
+    Process caution admonition nodes by creating Notion Callout blocks.
+    """
+    del section_level
+    return _create_admonition_callout(
+        node=node,
+        emoji="⚠️",
+        background_color=BGColor.YELLOW,
+    )
+
+
+@beartype
+@_process_node_to_blocks.register
+def _(
+    node: nodes.danger,
+    *,
+    section_level: int,
+) -> list[Block]:
+    """
+    Process danger admonition nodes by creating Notion Callout blocks.
+    """
+    del section_level
+    return _create_admonition_callout(
+        node=node,
+        emoji="🚨",
+        background_color=BGColor.RED,
+    )
+
+
+@beartype
+@_process_node_to_blocks.register
+def _(
+    node: nodes.error,
+    *,
+    section_level: int,
+) -> list[Block]:
+    """
+    Process error admonition nodes by creating Notion Callout blocks.
+    """
+    del section_level
+    return _create_admonition_callout(
+        node=node,
+        emoji="❌",
+        background_color=BGColor.RED,
+    )
+
+
+@beartype
+@_process_node_to_blocks.register
+def _(
+    node: nodes.hint,
+    *,
+    section_level: int,
+) -> list[Block]:
+    """
+    Process hint admonition nodes by creating Notion Callout blocks.
+    """
+    del section_level
+    return _create_admonition_callout(
+        node=node,
+        emoji="💡",
+        background_color=BGColor.GREEN,
+    )
+
+
+@beartype
+@_process_node_to_blocks.register
+def _(
+    node: nodes.important,
+    *,
+    section_level: int,
+) -> list[Block]:
+    """
+    Process important admonition nodes by creating Notion Callout blocks.
+    """
+    del section_level
+    return _create_admonition_callout(
+        node=node,
+        emoji="❗",
+        background_color=BGColor.RED,
+    )
+
+
+@beartype
+@_process_node_to_blocks.register
+def _(
+    node: nodes.admonition,
+    *,
+    section_level: int,
+) -> list[Block]:
+    """Process generic admonition nodes by creating Notion Callout blocks.
+
+    Generic admonitions have a title as the first child, followed by
+    content. The title becomes the callout text, and all content becomes
+    nested blocks.
+    """
+    del section_level
+
+    # Extract the title from the first child (admonitions always have title
+    # as first child)
+    title_node = node.children[0]
+    assert isinstance(title_node, nodes.title)
+    title_text = title_node.astext()
+    # All remaining children become nested blocks
+    content_children = node.children[1:]
+
+    block = UnoCallout(
+        text=text(text=title_text),
+        icon=Emoji(emoji="💬"),
+        color=BGColor.GRAY,
+    )
+
+    for child in content_children:
+        block.append(
+            blocks=_process_node_to_blocks(
+                child,
+                section_level=1,
+            )
+        )
+
+    return [block]
+
+
+@beartype
 @_process_node_to_blocks.register
 def _(
     node: CollapseNode,
@@ -725,6 +961,7 @@ def _(
     return [toggle_block]
 
 
+@beartype
 @_process_node_to_blocks.register
 def _(
     node: nodes.image,
@@ -747,6 +984,7 @@ def _(
     return [UnoImage(file=ExternalFile(url=image_url), caption=None)]
 
 
+@beartype
 @_process_node_to_blocks.register
 def _(
     node: video_node,
@@ -781,6 +1019,7 @@ def _(
     ]
 
 
+@beartype
 @_process_node_to_blocks.register
 def _(
     node: audio_node,
@@ -803,6 +1042,30 @@ def _(
     return [UnoAudio(file=ExternalFile(url=audio_url))]
 
 
+@beartype
+@_process_node_to_blocks.register
+def _(
+    node: _PdfNode,
+    *,
+    section_level: int,
+) -> list[Block]:
+    """Process PDF nodes by creating Notion PDF blocks.
+
+    This handles nodes created by our custom NotionPdfIncludeDirective.
+    """
+    del section_level
+
+    pdf_url = node.attributes["uri"]
+
+    if "://" not in pdf_url:
+        assert node.document is not None
+        abs_path = Path(node.document.settings.env.srcdir) / pdf_url
+        pdf_url = abs_path.as_uri()
+
+    return [UnoPDF(file=ExternalFile(url=pdf_url))]
+
+
+@beartype
 @_process_node_to_blocks.register
 def _(
     node: nodes.container,
@@ -812,32 +1075,41 @@ def _(
     """
     Process container nodes, especially for ``literalinclude`` with captions.
     """
-    del section_level
+    num_children_for_captioned_literalinclude = 2
+    if (
+        len(node.children) == num_children_for_captioned_literalinclude
+        and isinstance(node.children[0], nodes.caption)
+        and isinstance(node.children[1], nodes.literal_block)
+    ):
+        caption_node, literal_node = node.children
+        assert isinstance(caption_node, nodes.caption)
+        assert isinstance(literal_node, nodes.literal_block)
+        caption_rich_text = _create_rich_text_from_children(node=caption_node)
 
-    caption_node, literal_node = node.children
-    msg = (
-        "The only supported container type is a literalinclude with a caption"
-    )
-    assert isinstance(caption_node, nodes.caption), msg
-    assert isinstance(literal_node, nodes.literal_block), msg
-
-    caption_rich_text = _create_rich_text_from_children(node=caption_node)
-
-    code_text = _create_rich_text_from_children(node=literal_node)
-    pygments_lang = literal_node.get(key="language", failobj="")
-    language = _map_pygments_to_notion_language(
-        pygments_lang=pygments_lang,
-    )
-
-    return [
-        UnoCode(
-            text=code_text,
-            language=language,
-            caption=caption_rich_text,
+        code_text = _create_rich_text_from_children(node=literal_node)
+        pygments_lang = literal_node.get(key="language", failobj="")
+        language = _map_pygments_to_notion_language(
+            pygments_lang=pygments_lang,
         )
-    ]
+
+        return [
+            UnoCode(
+                text=code_text,
+                language=language,
+                caption=caption_rich_text,
+            )
+        ]
+
+    blocks: list[Block] = []
+    for child in node.children:
+        child_blocks = _process_node_to_blocks(
+            child, section_level=section_level
+        )
+        blocks.extend(child_blocks)
+    return blocks
 
 
+@beartype
 @_process_node_to_blocks.register
 def _(
     node: nodes.comment,
@@ -854,7 +1126,22 @@ def _(
 
 
 @beartype
-class NotionTranslator(NodeVisitor):  # pylint: disable=too-many-public-methods
+@_process_node_to_blocks.register
+def _(
+    node: nodes.document,
+    *,
+    section_level: int,
+) -> list[Block]:
+    """
+    Process document nodes by ignoring them completely.
+    """
+    del node
+    del section_level
+    return []
+
+
+@beartype
+class NotionTranslator(NodeVisitor):
     """
     Translate ``docutils`` nodes to Notion JSON.
     """
@@ -869,23 +1156,21 @@ class NotionTranslator(NodeVisitor):  # pylint: disable=too-many-public-methods
         self.body: str
         self._section_level = 0
 
-    def visit_title(self, node: nodes.Element) -> None:
+    def dispatch_visit(self, node: nodes.Node) -> None:
         """
-        Handle title nodes by creating appropriate Notion heading blocks.
+        Handle nodes by creating appropriate Notion heading blocks.
         """
+        if isinstance(node, nodes.section):
+            self._section_level += 1
+            return
+
         blocks = _process_node_to_blocks(
             node,
             section_level=self._section_level,
         )
         self._blocks.extend(blocks)
-        raise nodes.SkipNode
-
-    def visit_section(self, node: nodes.Element) -> None:
-        """
-        Handle section nodes by increasing the section level.
-        """
-        del node
-        self._section_level += 1
+        if not isinstance(node, nodes.document):
+            raise nodes.SkipNode
 
     def depart_section(self, node: nodes.Element) -> None:
         """
@@ -894,236 +1179,6 @@ class NotionTranslator(NodeVisitor):  # pylint: disable=too-many-public-methods
         del node
         self._section_level -= 1
 
-    def visit_paragraph(self, node: nodes.Element) -> None:
-        """
-        Handle paragraph nodes by creating Notion Paragraph blocks.
-        """
-        blocks = _process_node_to_blocks(
-            node,
-            section_level=self._section_level,
-        )
-        self._blocks.extend(blocks)
-        raise nodes.SkipNode
-
-    def visit_block_quote(self, node: nodes.Element) -> None:
-        """
-        Handle block quote nodes by creating Notion Quote blocks.
-        """
-        blocks = _process_node_to_blocks(
-            node,
-            section_level=self._section_level,
-        )
-        self._blocks.extend(blocks)
-
-        raise nodes.SkipNode
-
-    def visit_literal_block(self, node: nodes.Element) -> None:
-        """
-        Handle literal block nodes by creating Notion Code blocks.
-        """
-        blocks = _process_node_to_blocks(
-            node,
-            section_level=self._section_level,
-        )
-        self._blocks.extend(blocks)
-        raise nodes.SkipNode
-
-    def visit_bullet_list(self, node: nodes.Element) -> None:
-        """
-        Handle bullet list nodes by processing each list item.
-        """
-        blocks = _process_node_to_blocks(
-            node,
-            section_level=self._section_level,
-        )
-        self._blocks.extend(blocks)
-        raise nodes.SkipNode
-
-    def visit_enumerated_list(self, node: nodes.Element) -> None:
-        """
-        Handle enumerated list nodes by processing each list item.
-        """
-        blocks = _process_node_to_blocks(
-            node,
-            section_level=self._section_level,
-        )
-        self._blocks.extend(blocks)
-        raise nodes.SkipNode
-
-    def visit_topic(self, node: nodes.Element) -> None:
-        """
-        Handle topic nodes, specifically for table of contents.
-        """
-        blocks = _process_node_to_blocks(
-            node,
-            section_level=self._section_level,
-        )
-        self._blocks.extend(blocks)
-        raise nodes.SkipNode
-
-    def visit_note(self, node: nodes.Element) -> None:
-        """
-        Handle note admonition nodes by creating Notion Callout blocks.
-        """
-        blocks = _process_node_to_blocks(
-            node,
-            section_level=self._section_level,
-        )
-        self._blocks.extend(blocks)
-        raise nodes.SkipNode
-
-    def visit_warning(self, node: nodes.Element) -> None:
-        """
-        Handle warning admonition nodes by creating Notion Callout blocks.
-        """
-        blocks = _process_node_to_blocks(
-            node,
-            section_level=self._section_level,
-        )
-        self._blocks.extend(blocks)
-
-        raise nodes.SkipNode
-
-    def visit_tip(self, node: nodes.Element) -> None:
-        """
-        Handle tip admonition nodes by creating Notion Callout blocks.
-        """
-        blocks = _process_node_to_blocks(
-            node,
-            section_level=self._section_level,
-        )
-        self._blocks.extend(blocks)
-
-        raise nodes.SkipNode
-
-    def visit_table(self, node: nodes.Element) -> None:
-        """
-        Handle table nodes by creating Notion Table blocks.
-        """
-        blocks = _process_node_to_blocks(
-            node,
-            section_level=self._section_level,
-        )
-        self._blocks.extend(blocks)
-
-        raise nodes.SkipNode
-
-    def visit_compound(self, node: nodes.Element) -> None:
-        """
-        Handle compound admonition nodes by creating a table of contents block.
-        """
-        blocks = _process_node_to_blocks(
-            node,
-            section_level=self._section_level,
-        )
-        self._blocks.extend(blocks)
-
-        raise nodes.SkipNode
-
-    def visit_CollapseNode(self, node: nodes.Element) -> None:  # pylint: disable=invalid-name  # noqa: N802
-        """
-        Handle collapse nodes by creating Notion ToggleItem blocks.
-        """
-        blocks = _process_node_to_blocks(
-            node,
-            section_level=self._section_level,
-        )
-        self._blocks.extend(blocks)
-
-        raise nodes.SkipNode
-
-    def visit_image(self, node: nodes.Element) -> None:
-        """
-        Handle image nodes by creating Notion Image blocks.
-        """
-        blocks = _process_node_to_blocks(
-            node,
-            section_level=self._section_level,
-        )
-        self._blocks.extend(blocks)
-
-        raise nodes.SkipNode
-
-    def visit_video(self, node: nodes.Element) -> None:
-        """
-        Process a video node into Notion blocks.
-        """
-        blocks = _process_node_to_blocks(
-            node,
-            section_level=self._section_level,
-        )
-        self._blocks.extend(blocks)
-
-    def visit_container(self, node: nodes.Element) -> None:
-        """
-        Handle container nodes.
-        """
-        blocks = _process_node_to_blocks(
-            node,
-            section_level=self._section_level,
-        )
-        self._blocks.extend(blocks)
-
-        raise nodes.SkipNode
-
-    def visit_comment(self, node: nodes.Element) -> None:
-        """
-        Handle comment nodes by ignoring them completely.
-        """
-        blocks = _process_node_to_blocks(
-            node,
-            section_level=self._section_level,
-        )
-        self._blocks.extend(blocks)
-
-        raise nodes.SkipNode
-
-    def visit_document(self, node: nodes.Element) -> None:
-        """
-        Initialize block collection at document start.
-        """
-        assert self
-        del node
-
-    def visit_audio(self, node: nodes.Element) -> None:
-        """
-        Handle audio nodes by creating Notion Audio blocks.
-        """
-        blocks = _process_node_to_blocks(
-            node,
-            section_level=self._section_level,
-        )
-        self._blocks.extend(blocks)
-
-    @staticmethod
-    def depart_audio(node: nodes.Element) -> None:
-        """
-        Depart from audio nodes.
-        """
-        del node
-
-    @beartype
-    def _serialize_blocks(
-        self,
-        *,
-        blocks: list[Block],
-    ) -> list[dict[str, Any]]:
-        """
-        Convert the block tree to a JSON-serializable format, ignoring IDs from
-        tuples.
-        """
-        result: list[dict[str, Any]] = []
-        for block in blocks:
-            serialized_obj = block.obj_ref.serialize_for_api()
-            if isinstance(block, ChildrenMixin) and block.children:
-                serialized_obj[block.obj_ref.type]["children"] = (
-                    self._serialize_blocks(
-                        blocks=list(block.children),
-                    )
-                )
-            result.append(serialized_obj)
-        return result
-
     def depart_document(self, node: nodes.Element) -> None:
         """
         Output collected block tree as JSON at document end.
@@ -1131,7 +1186,10 @@ class NotionTranslator(NodeVisitor):  # pylint: disable=too-many-public-methods
         del node
 
         json_output = json.dumps(
-            obj=self._serialize_blocks(blocks=self._blocks),
+            obj=[
+                _serialize_block_with_children(block=block)
+                for block in self._blocks
+            ],
             indent=2,
             ensure_ascii=False,
         )
@@ -1149,26 +1207,36 @@ class NotionBuilder(TextBuilder):
 
 
 @beartype
-def _visit_video_node_notion(
-    translator: NotionTranslator,
-    node: video_node,
+def _notion_register_pdf_include_directive(
+    app: Sphinx,
 ) -> None:
     """
-    Visit a video node and process it into Notion blocks.
+    Register the PDF include directive.
     """
-    translator.visit_video(node=node)
+    if isinstance(app.builder, NotionBuilder):
+        sphinx_docutils.register_directive(
+            name="pdf-include",
+            directive=_NotionPdfIncludeDirective,
+        )
 
 
 @beartype
-def _depart_video_node_notion(
-    translator: NotionTranslator,
-    node: video_node,
-) -> None:
+def _filter_ulem(record: logging.LogRecord) -> bool:
+    """Filter out the warning about the `ulem package already being included`.
+
+    This warning is emitted by ``sphinxcontrib-text-styles`` or
+    ``sphinxnotes.strike`` when the ``ulem`` package is already included.
+
+    Our users may use both of these extensions, so we filter out the
+    warning.
+
+    See:
+
+    * https://github.com/sphinx-notes/strike/pull/10
+    * https://github.com/martinpriestley/sphinxcontrib-text-styles/pull/1
     """
-    Depart from a video node (no action needed).
-    """
-    del translator
-    del node
+    msg = record.getMessage()
+    return msg != "latex package 'ulem' already included"
 
 
 @beartype
@@ -1179,10 +1247,13 @@ def setup(app: Sphinx) -> ExtensionMetadata:
     app.add_builder(builder=NotionBuilder)
     app.set_translator(name="notion", translator_class=NotionTranslator)
 
-    app.add_node(
-        node=video_node,
-        notion=(_visit_video_node_notion, _depart_video_node_notion),
-        override=True,
+    app.connect(
+        event="builder-inited",
+        callback=_notion_register_pdf_include_directive,
     )
+
+    logger = logging.getLogger(name="sphinx.sphinx.registry")
+    logger.addFilter(filter=_filter_ulem)
+
     sphinxnotes.strike.SUPPORTED_BUILDERS.append(NotionBuilder)
     return {"parallel_read_safe": True}
