@@ -6,16 +6,18 @@ import base64
 import json
 import re
 import textwrap
-from collections.abc import Callable
+from collections.abc import Callable, Collection
 from pathlib import Path
 from typing import Any
 
+import anstrip
 import pytest
 from beartype import beartype
 from sphinx.testing.util import SphinxTestApp
 from ultimate_notion import Emoji
+from ultimate_notion.blocks import PDF as UnoPDF  # noqa: N811
 from ultimate_notion.blocks import Audio as UnoAudio
-from ultimate_notion.blocks import Block, ChildrenMixin
+from ultimate_notion.blocks import Block, ParentBlock
 from ultimate_notion.blocks import BulletedItem as UnoBulletedItem
 from ultimate_notion.blocks import Callout as UnoCallout
 from ultimate_notion.blocks import Code as UnoCode
@@ -40,12 +42,13 @@ from ultimate_notion.blocks import Table as UnoTable
 from ultimate_notion.blocks import (
     TableOfContents as UnoTableOfContents,
 )
+from ultimate_notion.blocks import ToDoItem as UnoToDoItem
 from ultimate_notion.blocks import (
     ToggleItem as UnoToggleItem,
 )
 from ultimate_notion.blocks import Video as UnoVideo
 from ultimate_notion.file import ExternalFile
-from ultimate_notion.obj_api.enums import BGColor, CodeLang
+from ultimate_notion.obj_api.enums import BGColor, CodeLang, Color
 from ultimate_notion.rich_text import text
 
 
@@ -58,7 +61,7 @@ def _details_from_block(
     Create a serialized block details from a Block.
     """
     serialized_obj = block.obj_ref.serialize_for_api()
-    if isinstance(block, ChildrenMixin) and block.children:
+    if isinstance(block, ParentBlock) and block.children:
         serialized_obj[block.obj_ref.type]["children"] = [
             _details_from_block(block=child) for child in block.children
         ]
@@ -74,7 +77,8 @@ def _assert_rst_converts_to_notion_objects(
     tmp_path: Path,
     extensions: tuple[str, ...] = ("sphinx_notion",),
     conf_py_content: str = "",
-) -> None:
+    expected_warnings: Collection[str] = (),
+) -> SphinxTestApp:
     """
     ReStructuredText content converts to expected Notion objects via Sphinx
     build process.
@@ -94,6 +98,16 @@ def _assert_rst_converts_to_notion_objects(
         confoverrides={"extensions": list(extensions)},
     )
     app.build()
+    assert app.statuscode == 0
+
+    warning_output = app.warning.getvalue()
+    ansi_stripped_warning_output = anstrip.strip(string=warning_output)
+    warnings = [
+        item.strip()
+        for item in ansi_stripped_warning_output.split(sep="WARNING: ")
+        if item.strip()
+    ]
+    assert list(expected_warnings) == warnings
 
     output_file = app.outdir / "index.json"
     with output_file.open(encoding="utf-8") as f:
@@ -105,6 +119,7 @@ def _assert_rst_converts_to_notion_objects(
     ]
 
     assert generated_json == expected_json
+    return app
 
 
 def test_single_paragraph(
@@ -743,6 +758,12 @@ def test_bullet_list_with_inline_formatting(
         ("note", "📝", BGColor.BLUE, "This is an important note."),
         ("warning", "⚠️", BGColor.YELLOW, "This is a warning message."),
         ("tip", "💡", BGColor.GREEN, "This is a helpful tip."),
+        ("attention", "👀", BGColor.YELLOW, "This requires your attention."),
+        ("caution", "⚠️", BGColor.YELLOW, "This is a caution message."),
+        ("danger", "🚨", BGColor.RED, "This is a danger message."),
+        ("error", "❌", BGColor.RED, "This is an error message."),
+        ("hint", "💡", BGColor.GREEN, "This is a helpful hint."),
+        ("important", "❗", BGColor.RED, "This is important information."),
     ],
 )
 def test_admonition_single_line(
@@ -970,6 +991,52 @@ def test_admonition_with_bullet_points(
     )
 
 
+def test_generic_admonition(
+    *,
+    make_app: Callable[..., SphinxTestApp],
+    tmp_path: Path,
+) -> None:
+    """Generic admonitions set callout text to the first line of the callout.
+
+    Generic admonitions require a title so are different from other
+    admonitions.
+    """
+    rst_content = """
+        .. admonition:: Important Information
+
+           This is the first paragraph.
+
+           This is the second paragraph.
+    """
+
+    callout = UnoCallout(
+        text=text(text="Important Information"),
+        icon=Emoji(emoji="💬"),
+        color=BGColor.GRAY,
+    )
+
+    nested_paragraph1 = UnoParagraph(
+        text=text(text="This is the first paragraph.")
+    )
+    nested_paragraph2 = UnoParagraph(
+        text=text(text="This is the second paragraph.")
+    )
+
+    callout.append(blocks=[nested_paragraph1])
+    callout.append(blocks=[nested_paragraph2])
+
+    expected_objects: list[Block] = [
+        callout,
+    ]
+
+    _assert_rst_converts_to_notion_objects(
+        rst_content=rst_content,
+        expected_objects=expected_objects,
+        make_app=make_app,
+        tmp_path=tmp_path,
+    )
+
+
 def test_nested_bullet_list(
     *,
     make_app: Callable[..., SphinxTestApp],
@@ -1716,37 +1783,6 @@ def test_strikethrough_text(
     )
 
 
-def test_bullet_list_item_invalid_nested_child_error(
-    *,
-    make_app: Callable[..., SphinxTestApp],
-    tmp_path: Path,
-) -> None:
-    """
-    Bullet list items with invalid nested children raise a clear error message.
-    """
-    rst_content = """
-        * First bullet point
-
-          Some paragraph that should not be here
-
-          * Nested bullet
-    """
-
-    index_rst = tmp_path / "src" / "index.rst"
-    expected_message = (
-        r"^The only thing Notion supports within a bullet list is a "
-        r"bullet list. Given paragraph on line 3 "
-        rf"in {re.escape(pattern=str(object=index_rst))}$"
-    )
-    with pytest.raises(expected_exception=ValueError, match=expected_message):
-        _assert_rst_converts_to_notion_objects(
-            rst_content=rst_content,
-            expected_objects=[],
-            make_app=make_app,
-            tmp_path=tmp_path,
-        )
-
-
 def test_comment_ignored(
     *,
     make_app: Callable[..., SphinxTestApp],
@@ -1849,7 +1885,7 @@ def test_list_table_header_maximum_one_allowed(
     tmp_path: Path,
 ) -> None:
     """
-    List table with header-rows option other than 0 or 1 raises ValueError.
+    List table with header-rows option other than 0 or 1 emits a warning.
     """
     rst_content = """
         .. list-table::
@@ -1863,19 +1899,29 @@ def test_list_table_header_maximum_one_allowed(
              - Cell a 2
     """
 
-    expected_message = (
-        r"^Tables with multiple header rows are not supported. "
-        r"First header row is on line 4 in "
-        rf"{re.escape(pattern=str(object=tmp_path / 'src' / 'index.rst'))}, "
-        r"last header row is on line 6"
+    expected_warning = (
+        "Tables with multiple header rows are not supported. "
+        f"First header row is on line 4 in {tmp_path / 'src' / 'index.rst'}, "
+        "last header row is on line 6"
+    )
+
+    table = UnoTable(n_rows=3, n_cols=2, header_row=True)
+    table[0, 0] = text(text="Header a 1")
+    table[0, 1] = text(text="Header a 2")
+    table[1, 0] = text(text="Header b 1")
+    table[1, 1] = text(text="Header b 2")
+    table[2, 0] = text(text="Cell a 1")
+    table[2, 1] = text(text="Cell a 2")
+
+    expected_objects: list[Block] = [table]
+
+    _assert_rst_converts_to_notion_objects(
+        rst_content=rst_content,
+        expected_objects=expected_objects,
+        make_app=make_app,
+        tmp_path=tmp_path,
+        expected_warnings=[expected_warning],
     )
-    with pytest.raises(expected_exception=ValueError, match=expected_message):
-        _assert_rst_converts_to_notion_objects(
-            rst_content=rst_content,
-            expected_objects=[],
-            make_app=make_app,
-            tmp_path=tmp_path,
-        )
 
 
 def test_list_table_stub_columns_one(
@@ -1932,7 +1978,7 @@ def test_list_table_stub_columns_two(
     tmp_path: Path,
 ) -> None:
     """
-    List table with :stub-columns: 2 raises ValueError.
+    List table with :stub-columns: 2 emits a warning.
     """
     rst_content = """
         .. list-table::
@@ -1950,17 +1996,31 @@ def test_list_table_stub_columns_two(
              - Row 2, Col 3
     """
 
-    expected_message = (
-        r"^Tables with more than 1 stub column are not supported. "
-        r"Found 2 stub columns.$"
+    expected_warning = (
+        "Tables with more than 1 stub column are not supported. "
+        "Found 2 stub columns."
+    )
+
+    table = UnoTable(n_rows=3, n_cols=3, header_row=True, header_col=True)
+    table[0, 0] = text(text="Header 1")
+    table[0, 1] = text(text="Header 2")
+    table[0, 2] = text(text="Header 3")
+    table[1, 0] = text(text="Row 1, Col 1")
+    table[1, 1] = text(text="Row 1, Col 2")
+    table[1, 2] = text(text="Row 1, Col 3")
+    table[2, 0] = text(text="Row 2, Col 1")
+    table[2, 1] = text(text="Row 2, Col 2")
+    table[2, 2] = text(text="Row 2, Col 3")
+
+    expected_objects: list[Block] = [table]
+
+    _assert_rst_converts_to_notion_objects(
+        rst_content=rst_content,
+        expected_objects=expected_objects,
+        make_app=make_app,
+        tmp_path=tmp_path,
+        expected_warnings=[expected_warning],
     )
-    with pytest.raises(expected_exception=ValueError, match=expected_message):
-        _assert_rst_converts_to_notion_objects(
-            rst_content=rst_content,
-            expected_objects=[],
-            make_app=make_app,
-            tmp_path=tmp_path,
-        )
 
 
 def test_list_table_with_title_error(
@@ -1969,7 +2029,7 @@ def test_list_table_with_title_error(
     tmp_path: Path,
 ) -> None:
     """
-    List table with title raises ValueError since Notion tables do not have
+    List table with title emits a warning since Notion tables do not have
     titles.
     """
     rst_content = """
@@ -1982,16 +2042,629 @@ def test_list_table_with_title_error(
              - Cell 2
     """
 
-    expected_message = (
-        r"^Table has a title 'My Table Title' on line 1 in "
-        rf"{re.escape(pattern=str(object=tmp_path / 'src' / 'index.rst'))}, "
-        r"but Notion tables do not "
-        r"have titles.$"
+    expected_warning = (
+        f"Table has a title 'My Table Title' on line 1 in "
+        f"{tmp_path / 'src' / 'index.rst'}, "
+        "but Notion tables do not have titles."
+    )
+
+    table = UnoTable(n_rows=2, n_cols=2, header_row=True)
+    table[0, 0] = text(text="Header 1")
+    table[0, 1] = text(text="Header 2")
+    table[1, 0] = text(text="Cell 1")
+    table[1, 1] = text(text="Cell 2")
+
+    expected_objects: list[Block] = [table]
+
+    _assert_rst_converts_to_notion_objects(
+        rst_content=rst_content,
+        expected_objects=expected_objects,
+        make_app=make_app,
+        tmp_path=tmp_path,
+        expected_warnings=[expected_warning],
+    )
+
+
+@pytest.mark.parametrize(
+    argnames="extensions",
+    argvalues=[
+        ("sphinx_notion",),
+        ("sphinx_notion", "sphinx_simplepdf"),
+        ("sphinx_simplepdf", "sphinx_notion"),
+    ],
+)
+def test_simple_pdf(
+    *,
+    make_app: Callable[..., SphinxTestApp],
+    tmp_path: Path,
+    extensions: tuple[str, ...],
+) -> None:
+    """
+    ``pdf-include`` directives become Notion PDF blocks with URL.
+    """
+    rst_content = """
+        .. pdf-include:: https://www.example.com/path/to/document.pdf
+    """
+
+    expected_objects: list[Block] = [
+        UnoPDF(
+            file=ExternalFile(
+                url="https://www.example.com/path/to/document.pdf"
+            )
+        ),
+    ]
+
+    _assert_rst_converts_to_notion_objects(
+        rst_content=rst_content,
+        expected_objects=expected_objects,
+        make_app=make_app,
+        tmp_path=tmp_path,
+        extensions=extensions,
+    )
+
+
+def test_pdf_with_options(
+    *,
+    make_app: Callable[..., SphinxTestApp],
+    tmp_path: Path,
+) -> None:
+    """
+    PDF directives with options (width, height) are processed correctly.
+    """
+    rst_content = """
+        .. pdf-include:: https://www.example.com/path/to/document.pdf
+           :width: 50%
+           :height: 300px
+    """
+
+    expected_objects: list[Block] = [
+        UnoPDF(
+            file=ExternalFile(
+                url="https://www.example.com/path/to/document.pdf"
+            ),
+        ),
+    ]
+
+    _assert_rst_converts_to_notion_objects(
+        rst_content=rst_content,
+        expected_objects=expected_objects,
+        make_app=make_app,
+        tmp_path=tmp_path,
+        extensions=("sphinx_notion",),
+    )
+
+
+def test_local_pdf_file(
+    *,
+    make_app: Callable[..., SphinxTestApp],
+    tmp_path: Path,
+) -> None:
+    """
+    Local PDF files are converted to file:// URLs in the JSON output.
+    """
+    srcdir = tmp_path / "src"
+    srcdir.mkdir()
+    test_pdf_path = srcdir / "test_document.pdf"
+    # Create a minimal PDF file (just some dummy data)
+    test_pdf_path.write_bytes(data=b"fake pdf content")
+
+    rst_content = """
+        .. pdf-include:: test_document.pdf
+    """
+
+    expected_objects: list[Block] = [
+        UnoPDF(file=ExternalFile(url=test_pdf_path.as_uri())),
+    ]
+
+    _assert_rst_converts_to_notion_objects(
+        rst_content=rst_content,
+        expected_objects=expected_objects,
+        make_app=make_app,
+        tmp_path=tmp_path,
+        extensions=("sphinx_notion",),
+    )
+
+
+@pytest.mark.parametrize(
+    argnames="extensions",
+    argvalues=[
+        ("sphinx_notion", "sphinx_simplepdf"),
+        ("sphinx_simplepdf", "sphinx_notion"),
+    ],
+)
+def test_pdf_with_html(
+    *,
+    make_app: Callable[..., SphinxTestApp],
+    tmp_path: Path,
+    extensions: tuple[str, ...],
+) -> None:
+    """
+    PDF directives with HTML output are processed correctly.
+    """
+    rst_content = """
+        .. pdf-include:: https://www.example.com/path/to/document.pdf
+    """
+    srcdir = tmp_path / "src"
+    srcdir.mkdir()
+    (srcdir / "conf.py").touch()
+    test_pdf_path = srcdir / "test_document.pdf"
+    # Create a minimal PDF file (just some dummy data)
+    test_pdf_path.write_bytes(data=b"fake pdf content")
+    (srcdir / "index.rst").write_text(data=rst_content)
+    app = make_app(
+        srcdir=srcdir,
+        builddir=tmp_path / "build",
+        buildername="html",
+        confoverrides={"extensions": list(extensions)},
+    )
+    app.build()
+    assert app.statuscode == 0
+    assert not app.warning.getvalue()
+    index_html = (tmp_path / "build" / "html" / "index.html").read_text()
+    expected_iframe = (
+        "<iframe "
+        'src="https://www.example.com/path/to/document.pdf" '
+        'style="height: 400px; width: 100%">'
+        "</iframe>"
+    )
+    assert expected_iframe in index_html
+
+
+def test_colored_text(
+    *,
+    make_app: Callable[..., SphinxTestApp],
+    tmp_path: Path,
+) -> None:
+    """
+    Colored text from ``sphinxcontrib-text-styles`` becomes rich text.
+    """
+    rst_content = """
+        This is :text-red:`red text` and :text-blue:`blue text` \
+and :text-green:`green text`.
+    """
+
+    normal_text = text(text="This is ")
+    red_text = text(text="red text", color=Color.RED)
+    normal_text2 = text(text=" and ")
+    blue_text = text(text="blue text", color=Color.BLUE)
+    normal_text3 = text(text=" and ")
+    green_text = text(text="green text", color=Color.GREEN)
+    normal_text4 = text(text=".")
+
+    combined_text = (
+        normal_text
+        + red_text
+        + normal_text2
+        + blue_text
+        + normal_text3
+        + green_text
+        + normal_text4
+    )
+
+    expected_paragraph = UnoParagraph(text=combined_text)
+
+    expected_objects: list[Block] = [expected_paragraph]
+
+    _assert_rst_converts_to_notion_objects(
+        rst_content=rst_content,
+        expected_objects=expected_objects,
+        make_app=make_app,
+        tmp_path=tmp_path,
+        extensions=("sphinx_notion", "sphinxcontrib_text_styles"),
+    )
+
+
+@pytest.mark.parametrize(
+    argnames=("color_name", "expected_color"),
+    argvalues=[
+        ("red", Color.RED),
+        ("blue", Color.BLUE),
+        ("green", Color.GREEN),
+        ("yellow", Color.YELLOW),
+        ("orange", Color.ORANGE),
+        ("purple", Color.PURPLE),
+        ("pink", Color.PINK),
+        ("brown", Color.BROWN),
+        ("gray", Color.GRAY),
+    ],
+)
+def test_individual_colors(
+    *,
+    make_app: Callable[..., SphinxTestApp],
+    tmp_path: Path,
+    color_name: str,
+    expected_color: Color,
+) -> None:
+    """
+    Each supported color is converted correctly.
+    """
+    rst_content = f"""
+        This is :text-{color_name}:`{color_name} text`.
+    """
+
+    normal_text = text(text="This is ")
+    colored_text = text(text=f"{color_name} text", color=expected_color)
+    normal_text2 = text(text=".")
+
+    combined_text = normal_text + colored_text + normal_text2
+
+    expected_paragraph = UnoParagraph(text=combined_text)
+
+    expected_objects: list[Block] = [expected_paragraph]
+
+    _assert_rst_converts_to_notion_objects(
+        rst_content=rst_content,
+        expected_objects=expected_objects,
+        make_app=make_app,
+        tmp_path=tmp_path,
+        extensions=("sphinx_notion", "sphinxcontrib_text_styles"),
+    )
+
+
+def test_text_styles_non_color(
+    *,
+    make_app: Callable[..., SphinxTestApp],
+    tmp_path: Path,
+) -> None:
+    """
+    Non-color text styles (like underline) emit a warning.
+    """
+    rst_content = """
+        This is :text-underline:`underlined text`.
+    """
+
+    expected_warning = (
+        "Unsupported text style classes: text-underline. "
+        "Text will be rendered without styling."
+    )
+
+    normal_text = text(text="This is ")
+    underline_text = text(text="underlined text")
+    normal_text2 = text(text=".")
+
+    combined_text = normal_text + underline_text + normal_text2
+
+    expected_paragraph = UnoParagraph(text=combined_text)
+
+    expected_objects: list[Block] = [expected_paragraph]
+
+    _assert_rst_converts_to_notion_objects(
+        rst_content=rst_content,
+        expected_objects=expected_objects,
+        make_app=make_app,
+        tmp_path=tmp_path,
+        extensions=("sphinx_notion", "sphinxcontrib_text_styles"),
+        expected_warnings=[expected_warning],
+    )
+
+
+def test_inline_node_without_classes(
+    *,
+    make_app: Callable[..., SphinxTestApp],
+    tmp_path: Path,
+) -> None:
+    """
+    Inline nodes without classes are handled as plain text.
+    """
+    # Using a custom role to create an inline node without classes
+    conf_py_content = """
+from docutils import nodes
+
+def setup(app):
+    app.add_role(
+        'custom',
+        lambda name, rawtext, text, lineno, inliner, options={}, content=[]:
+            ([nodes.inline(rawtext, text)], [])
+    )
+    """
+
+    rst_content = """
+        This is :custom:`custom text`.
+    """
+
+    normal_text = text(text="This is ")
+    custom_text = text(text="custom text")
+    normal_text2 = text(text=".")
+
+    combined_text = normal_text + custom_text + normal_text2
+
+    expected_paragraph = UnoParagraph(text=combined_text)
+
+    expected_objects: list[Block] = [expected_paragraph]
+
+    _assert_rst_converts_to_notion_objects(
+        rst_content=rst_content,
+        expected_objects=expected_objects,
+        make_app=make_app,
+        tmp_path=tmp_path,
+        conf_py_content=conf_py_content,
+    )
+
+
+def test_text_styles_and_strike(
+    *,
+    make_app: Callable[..., SphinxTestApp],
+    tmp_path: Path,
+) -> None:
+    """There is no warning when using text styles and strike.
+
+    This demonstrates a workaround for an issue where the extensions
+    conflicted with each other.
+    """
+    rst_content = """
+        This is :text-red:`red text` and :strike:`strikethrough text`.
+    """
+
+    normal_text = text(text="This is ")
+    red_text = text(text="red text", color=Color.RED)
+    normal_text2 = text(text=" and ")
+    strikethrough_text = text(text="strikethrough text", strikethrough=True)
+    normal_text3 = text(text=".")
+
+    combined_text = (
+        normal_text
+        + red_text
+        + normal_text2
+        + strikethrough_text
+        + normal_text3
+    )
+
+    expected_paragraph = UnoParagraph(text=combined_text)
+
+    expected_objects: list[Block] = [expected_paragraph]
+
+    _assert_rst_converts_to_notion_objects(
+        rst_content=rst_content,
+        expected_objects=expected_objects,
+        make_app=make_app,
+        tmp_path=tmp_path,
+        extensions=(
+            "sphinx_notion",
+            "sphinxcontrib_text_styles",
+            "sphinxnotes.strike",
+        ),
+    )
+
+
+def test_flat_task_list(
+    *,
+    make_app: Callable[..., SphinxTestApp],
+    tmp_path: Path,
+) -> None:
+    """
+    Flat task lists become separate Notion ToDoItem blocks.
+    """
+    rst_content = """
+        .. task-list::
+
+           - [ ] Unchecked task item
+           - [x] Checked task item
+           - [ ] Another unchecked task with **bold text**
+    """
+    expected_objects: list[Block] = [
+        UnoToDoItem(text=text(text="Unchecked task item"), checked=False),
+        UnoToDoItem(text=text(text="Checked task item"), checked=True),
+        UnoToDoItem(
+            text=(
+                text(text="Another unchecked task with ", bold=False)
+                + text(text="bold text", bold=True)
+            ),
+            checked=False,
+        ),
+    ]
+    _assert_rst_converts_to_notion_objects(
+        rst_content=rst_content,
+        expected_objects=expected_objects,
+        make_app=make_app,
+        tmp_path=tmp_path,
+        extensions=("sphinx_notion", "sphinx_immaterial.task_lists"),
+    )
+
+
+def test_bullet_list_with_nested_content(
+    *,
+    make_app: Callable[..., SphinxTestApp],
+    tmp_path: Path,
+) -> None:
+    """
+    Test that bullet lists can contain nested content like paragraphs and
+    nested bullets.
+    """
+    rst_content = """
+        * First bullet point
+
+          This is a paragraph nested within a bullet list item.
+
+          * Nested bullet point
+          * Another nested bullet
+
+        * Second bullet point
+
+          Another nested paragraph.
+    """
+
+    first_bullet = UnoBulletedItem(text=text(text="First bullet point"))
+
+    nested_paragraph = UnoParagraph(
+        text=text(text="This is a paragraph nested within a bullet list item.")
+    )
+    first_bullet.append(blocks=[nested_paragraph])
+
+    nested_bullet_1 = UnoBulletedItem(text=text(text="Nested bullet point"))
+    nested_bullet_2 = UnoBulletedItem(text=text(text="Another nested bullet"))
+    first_bullet.append(blocks=[nested_bullet_1])
+    first_bullet.append(blocks=[nested_bullet_2])
+
+    second_bullet = UnoBulletedItem(text=text(text="Second bullet point"))
+
+    nested_paragraph_2 = UnoParagraph(
+        text=text(text="Another nested paragraph.")
+    )
+    second_bullet.append(blocks=[nested_paragraph_2])
+
+    expected_objects: list[Block] = [
+        first_bullet,
+        second_bullet,
+    ]
+    _assert_rst_converts_to_notion_objects(
+        rst_content=rst_content,
+        expected_objects=expected_objects,
+        make_app=make_app,
+        tmp_path=tmp_path,
+    )
+
+
+def test_task_list_with_nested_content(
+    *,
+    make_app: Callable[..., SphinxTestApp],
+    tmp_path: Path,
+) -> None:
+    """
+    Task lists with nested content should create ToDoItem blocks with nested
+    children.
+    """
+    rst_content = """
+        .. task-list::
+
+           - [ ] Task with nested content
+
+             This is a paragraph nested within the task item.
+
+             * A bullet point nested within the task item.
+    """
+
+    # Create the main task item
+    task_item = UnoToDoItem(
+        text=text(text="Task with nested content"), checked=False
+    )
+
+    # Add nested paragraph
+    nested_paragraph = UnoParagraph(
+        text=text(text="This is a paragraph nested within the task item.")
+    )
+    task_item.append(blocks=[nested_paragraph])
+
+    # Add nested bullet list
+    nested_bullet = UnoBulletedItem(
+        text=text(text="A bullet point nested within the task item.")
+    )
+    task_item.append(blocks=[nested_bullet])
+
+    expected_objects: list[Block] = [task_item]
+
+    _assert_rst_converts_to_notion_objects(
+        rst_content=rst_content,
+        expected_objects=expected_objects,
+        make_app=make_app,
+        tmp_path=tmp_path,
+        extensions=("sphinx_notion", "sphinx_immaterial.task_lists"),
+    )
+
+
+def test_nested_task_list(
+    *,
+    make_app: Callable[..., SphinxTestApp],
+    tmp_path: Path,
+) -> None:
+    """
+    Nested task lists should create nested ToDoItem blocks.
+    """
+    rst_content = """
+        .. task-list::
+
+           1. [x] Task A
+           2. [ ] Task B
+
+              .. task-list::
+
+                  * [x] Task B1
+                  * [x] Task B2
+                  * [ ] Task B3
+
+              A rogue paragraph with a reference to
+              the `parent task_list <task_list_example>`.
+
+              - A list item without a checkbox.
+              - [ ] Another bullet point.
+
+           3. [ ] Task C
+    """
+    # Create Task B with nested children (including the rogue paragraph)
+    task_b = UnoToDoItem(text=text(text="Task B"), checked=False)
+    task_b.append(
+        blocks=[UnoToDoItem(text=text(text="Task B1"), checked=True)]
+    )
+    task_b.append(
+        blocks=[UnoToDoItem(text=text(text="Task B2"), checked=True)]
+    )
+    task_b.append(
+        blocks=[UnoToDoItem(text=text(text="Task B3"), checked=False)]
+    )
+    # The rogue paragraph is nested within Task B
+    # Note: The actual output has the text split across multiple rich text
+    # segments
+    rogue_paragraph = UnoParagraph(
+        text=text(text="A rogue paragraph with a reference to\nthe ")
+        + text(text="parent task_list <task_list_example>")
+        + text(text=".")
+    )
+    task_b.append(blocks=[rogue_paragraph])
+
+    # Regular bullet list items should be nested within Task B as bullet items
+    regular_bullet = UnoBulletedItem(
+        text=text(text="A list item without a checkbox.")
+    )
+    task_b.append(blocks=[regular_bullet])
+
+    # Another bullet item (has "[ ]" but should be treated as a bullet)
+    another_bullet = UnoBulletedItem(
+        text=text(text="[ ] Another bullet point.")
+    )
+    task_b.append(blocks=[another_bullet])
+
+    expected_objects: list[Block] = [
+        UnoToDoItem(text=text(text="Task A"), checked=True),
+        task_b,
+        UnoToDoItem(text=text(text="Task C"), checked=False),
+    ]
+    _assert_rst_converts_to_notion_objects(
+        rst_content=rst_content,
+        expected_objects=expected_objects,
+        make_app=make_app,
+        tmp_path=tmp_path,
+        extensions=("sphinx_notion", "sphinx_immaterial.task_lists"),
+    )
+
+
+def test_task_list_quote(
+    *,
+    make_app: Callable[..., SphinxTestApp],
+    tmp_path: Path,
+) -> None:
+    """
+    A quote can exist within a task list.
+    """
+    rst_content = """
+    .. task-list::
+
+        1. [x] Task A
+        2. [ ] Task B
+
+          foo
+    """
+
+    # The actual processing creates a flat structure where the quote
+    # becomes a separate quote block
+    expected_objects: list[Block] = [
+        UnoToDoItem(text=text(text="Task A"), checked=True),
+        UnoToDoItem(text=text(text="Task B"), checked=False),
+        UnoQuote(text=text(text="foo")),
+    ]
+
+    _assert_rst_converts_to_notion_objects(
+        rst_content=rst_content,
+        expected_objects=expected_objects,
+        make_app=make_app,
+        tmp_path=tmp_path,
+        extensions=("sphinx_notion", "sphinx_immaterial.task_lists"),
     )
-    with pytest.raises(expected_exception=ValueError, match=expected_message):
-        _assert_rst_converts_to_notion_objects(
-            rst_content=rst_content,
-            expected_objects=[],
-            make_app=make_app,
-            tmp_path=tmp_path,
-        )
